@@ -51,6 +51,24 @@
 #define DEBUG_MSG(x)
 #endif
 
+static int gnome_authentication_manager_dialog_visible = 0;
+
+/**
+ * gnome_authentication_manager_dialog_is_visible:
+ *
+ * This function checks whether there are any references on the authentication dialog and returns a 
+ * a gboolean value.
+ *
+ * Returns: TRUE if there are any references on the authentication dialog, FALSE otherwise.
+ *
+ * Since: 2.8
+ */
+gboolean
+gnome_authentication_manager_dialog_is_visible (void)
+{
+	return gnome_authentication_manager_dialog_visible > 0;
+}
+
 static GnomePasswordDialog *
 construct_password_dialog (gboolean is_proxy_authentication, const GnomeVFSModuleCallbackAuthenticationIn *in_args)
 {
@@ -135,19 +153,12 @@ authentication_dialog_button_clicked (GtkDialog *dialog,
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
-static void
-authentication_dialog_closed (GtkDialog *dialog, CallbackInfo *info)
-{
-	DEBUG_MSG (("+%s\n", G_GNUC_FUNCTION));
-
-	gtk_widget_destroy (GTK_WIDGET (dialog));
-}
-
 static void
 authentication_dialog_destroyed (GtkDialog *dialog, CallbackInfo *info)
 {
 	DEBUG_MSG (("+%s\n", G_GNUC_FUNCTION));
 
+	gnome_authentication_manager_dialog_visible--;
 	mark_callback_completed (info);	
 }
 
@@ -179,14 +190,13 @@ present_authentication_dialog_nonblocking (CallbackInfo *info)
 	g_signal_connect (dialog, "response", 
 			  G_CALLBACK (authentication_dialog_button_clicked), info);
 
-	g_signal_connect (dialog, "close", 
-			  G_CALLBACK (authentication_dialog_closed), info);
-
 	g_signal_connect (dialog, "destroy", 
 			  G_CALLBACK (authentication_dialog_destroyed), info);
 
 	gtk_widget_show (GTK_WIDGET (dialog));
 
+	gnome_authentication_manager_dialog_visible++;
+	
 	return 0;
 }
 
@@ -285,6 +295,7 @@ construct_full_password_dialog (const GnomeVFSModuleCallbackFullAuthenticationIn
 		g_string_append_printf (name, "/%s", in_args->object);
 	}
 	if (in_args->domain != NULL) {
+		/* Translators: "You must log in to acces user@server.com/share domain MYWINDOWSDOMAIN." */
 		message = g_strdup_printf (_("You must log in to access %s domain %s\n"), name->str, in_args->domain);
 	} else {
 		message = g_strdup_printf (_("You must log in to access %s\n"), name->str);
@@ -395,6 +406,7 @@ full_authentication_dialog_destroyed (GtkDialog *dialog, FullCallbackInfo *info)
 
 	info->response (info->response_data);
 	g_free (info);
+	gnome_authentication_manager_dialog_visible--;
 }
 
 static gint /* GtkFunction */
@@ -425,14 +437,13 @@ present_full_authentication_dialog_nonblocking (FullCallbackInfo *info)
 	g_signal_connect (dialog, "response", 
 			  G_CALLBACK (full_authentication_dialog_button_clicked), info);
 
-	g_signal_connect (dialog, "close", 
-			  G_CALLBACK (authentication_dialog_closed), info);
-
 	g_signal_connect (dialog, "destroy", 
 			  G_CALLBACK (full_authentication_dialog_destroyed), info);
 
 	gtk_widget_show (GTK_WIDGET (dialog));
 
+	gnome_authentication_manager_dialog_visible++;
+	
 	return 0;
 }
 
@@ -565,7 +576,7 @@ vfs_async_fill_authentication_callback (gconstpointer in, size_t in_size,
 	info->out_args = out_real;
 	info->response = response;
 	info->response_data = response_data;
-	
+	/* Check this return? */	
 	request = gnome_keyring_find_network_password (in_real->username,
 						       in_real->domain,
 						       in_real->server,
@@ -678,7 +689,7 @@ vfs_async_save_authentication_callback (gconstpointer in, size_t in_size,
 	info->out_args = out_real;
 	info->response = response;
 	info->response_data = response_data;
-	
+	/* Check this return? */	
 	request = gnome_keyring_set_network_password (in_real->keyring,
 						      in_real->username,
 						      in_real->domain,
@@ -700,7 +711,6 @@ vfs_save_authentication_callback (gconstpointer in, size_t in_size,
 				  gpointer user_data)
 {
 	GnomeVFSModuleCallbackSaveAuthenticationIn *in_real;
-	GnomeVFSModuleCallbackSaveAuthenticationOut *out_real;
 	GnomeKeyringResult result;
 	guint32 item;
 	
@@ -711,10 +721,9 @@ vfs_save_authentication_callback (gconstpointer in, size_t in_size,
 	g_return_if_fail (out != NULL);
 
 	in_real = (GnomeVFSModuleCallbackSaveAuthenticationIn *)in;
-	out_real = (GnomeVFSModuleCallbackSaveAuthenticationOut *)out;
 
 	DEBUG_MSG (("+%s uri:'%s' \n", G_GNUC_FUNCTION, in_real->uri));
-
+	/* Check this return? */
 	result = gnome_keyring_set_network_password_sync (in_real->keyring,
 							  in_real->username,
 							  in_real->domain,
@@ -752,6 +761,7 @@ question_dialog_destroyed (GtkDialog *dialog, QuestionCallbackInfo *info)
 {
 	info->response (info->response_data);
 	g_free (info);
+	gnome_authentication_manager_dialog_visible--;
 }
 
 static void
@@ -813,6 +823,8 @@ present_question_dialog_nonblocking (QuestionCallbackInfo *info)
 			  G_CALLBACK (question_dialog_destroyed), info);
 	
 	gtk_widget_show (GTK_WIDGET (dialog));
+
+	gnome_authentication_manager_dialog_visible++;
 	return 0;
 }
 
@@ -846,7 +858,11 @@ vfs_async_question_callback (gconstpointer in, size_t in_size,
 	info->response = response;
 	info->response_data = response_data;
 
+	GDK_THREADS_ENTER ();
+
 	present_question_dialog_nonblocking (info);
+
+	GDK_THREADS_LEAVE ();
 }
 
 static void /* GnomeVFSModuleCallback */
@@ -867,18 +883,30 @@ vfs_question_callback (gconstpointer in, size_t in_size,
 	in_real = (GnomeVFSModuleCallbackQuestionIn *)in;
 	out_real = (GnomeVFSModuleCallbackQuestionOut *)out;
 	
+	GDK_THREADS_ENTER ();
+
 	out_real->answer = -1; /* Set a default value */
 	dialog = create_question_dialog (in_real->primary_message, in_real->secondary_message, in_real->choices);
 	out_real->answer = gtk_dialog_run (GTK_DIALOG(dialog));
 
 	gtk_widget_destroy (GTK_WIDGET (dialog));
+
+	GDK_THREADS_LEAVE ();
 }
 
-
-/* If you call this, and you use threads with gtk+, you must never
+/** 
+ * gnome_authentication_manager_init:
+ * 
+ * This function checks for thread support and does a thread initialisation, if the support 
+ * is available. Also sets the default sync and async gnome-vfs callbacks for various types
+ * of authentication.
+ * 
+ * Note: If you call this, and you use threads with gtk+, you must never
  * hold the gdk lock while doing synchronous gnome-vfs calls. Otherwise
  * an authentication callback presenting a dialog could try to grab the
  * already held gdk lock, causing a deadlock.
+ *
+ * Since: 2.4
  */
 void
 gnome_authentication_manager_init (void)
@@ -945,6 +973,14 @@ gnome_authentication_manager_init (void)
 
 }
 
+/**
+ * gnome_authentication_manager_push_async:
+ *
+ * This function calls gnome_vfs_async_module_callback_push() to set temporary async handlers for 
+ * the various types of authentication.
+ *  
+ * Since: 2.6
+ */
 void
 gnome_authentication_manager_push_async (void)
 {
@@ -975,6 +1011,14 @@ gnome_authentication_manager_push_async (void)
 					      NULL);
 }
 
+/**
+ * gnome_authentication_manager_pop_async:
+ *
+ * This function calls gnome_vfs_async_module_callback_pop() to remove all the temporary async 
+ * gnome-vfs callbacks associated with various types of authentication.
+ *
+ * Since: 2.6
+ */
 void
 gnome_authentication_manager_pop_async (void)
 {
@@ -986,6 +1030,14 @@ gnome_authentication_manager_pop_async (void)
 	gnome_vfs_async_module_callback_pop (GNOME_VFS_MODULE_CALLBACK_QUESTION);
 }
 
+/**
+ * gnome_authentication_manager_push_sync:
+ *
+ * This function calls gnome_vfs_module_callback_push() to set temperory handlers for 
+ * various types of authentication.
+ *
+ * Since: 2.6
+ */
 void
 gnome_authentication_manager_push_sync (void)
 {
@@ -1017,6 +1069,15 @@ gnome_authentication_manager_push_sync (void)
 					NULL);
 }
 
+/**
+ * gnome_authentication_manager_pop_sync:
+ * 
+ * This function calls gnome_vfs_module_callback_pop() to remove all the gnome-vfs sync handlers associated
+ * with various types of authentication.
+ *
+ * Since: 2.6
+ */
+
 void
 gnome_authentication_manager_pop_sync (void)
 {
